SystemOrganization addCategory: #'CT-MarkupBrowser'!
SystemOrganization addCategory: #'CT-MarkupBrowser-Nodes'!

OBPaneScroller subclass: #CtPaneScroller
	instanceVariableNames: 'firstColumnWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-MarkupBrowser'!

!CtPaneScroller commentStamp: 'mt 6/5/2009 15:51' prior: 0!
Just like OBPaneScroller but gives the first column a fixed space.!

!CtPaneScroller class methodsFor: 'as yet unclassified' stamp: 'mt 3/18/2010 19:16'!
animationDuration

	^ 200! !

!CtPaneScroller methodsFor: 'accessing' stamp: 'mt 6/5/2009 16:01'!
firstColumnWidth

	firstColumnWidth ifNil: [firstColumnWidth := (Preferences standardListFont widthOf: $x) * 15].
	^ firstColumnWidth! !

!CtPaneScroller methodsFor: 'accessing' stamp: 'mt 6/5/2009 15:55'!
firstColumnWidth: anObject

	firstColumnWidth := anObject.! !

!CtPaneScroller methodsFor: 'layout' stamp: 'mt 6/5/2009 15:59'!
paneWidthsToFit: limit
	| padded newLimit |
	padded := Array new: self paneCount.
	newLimit := limit - self firstColumnWidth.
	padded atAllPut: (newLimit / (self sizing - 1)) floor.
	padded at: 1 put: self firstColumnWidth.
	(1 to: limit - padded sum) do: [:i | padded at: i put: (padded at: i) + 1].
	^ padded! !

!OBClassCategoryNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 14:01'!
hasMarkups

	^ true! !

!OBClassCategoryNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 14:24'!
markupSourceName

	^ 'class category "{1}"' format: {self name}! !

!OBClassCategoryNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/2/2010 15:07:08.318'!
referencesForMarkups

	| result packageInfo |
	result := OrderedCollection new.
	
	packageInfo := PackageOrganizer default packageNamed: self name ifAbsent: [].
	
	(packageInfo
		ifNil: [self classNames collect: [:ea | environment at: ea]]
		ifNotNil: [packageInfo classes])
		do: [:c |
			c selectors do: [ :each |
				result add: (MethodReference class: c selector: each)].
			c class selectors do: [ :each |
				result add: (MethodReference class: c class selector: each)]].
	^ result! !

OBCodeBrowser subclass: #CtMarkupBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-MarkupBrowser'!

!CtMarkupBrowser class methodsFor: 'class initialization' stamp: 'mt 9/22/2010 16:01:30.084'!
initialize

	super initialize.
	
	(Preferences dictionaryOfPreferences includesKey: #rememberMarkupReads)
		ifFalse: [Preferences
			addBooleanPreference: #rememberMarkupReads
			category: #CodeTalk
			default: true
			balloonHelp: 'Shows markups differently in markup browser that have been already read.'].! !

!CtMarkupBrowser class methodsFor: 'as yet unclassified' stamp: 'mt 6/5/2009 15:12'!
maxPanes

	^ 2! !

!CtMarkupBrowser class methodsFor: 'as yet unclassified' stamp: 'mt 6/5/2009 15:12'!
minPanes

	^ 2! !

!CtMarkupBrowser class methodsFor: 'as yet unclassified' stamp: 'mt 6/5/2009 15:50'!
navigationPanel

	^ CtMarkupColumnPanel minPanes: self minPanes maxPanes: self maxPanes! !

!CtMarkupBrowser class methodsFor: 'as yet unclassified' stamp: 'mt 6/23/2009 14:56'!
openOn: aCodeNode

	| rootNode filterNode markupNode |
	rootNode := OBMetaNode named: 'environment'.
	filterNode := OBMetaNode named: 'filter'.
	filterNode ancestrySelector: #isDescendantOfMarkup:.
	
	markupNode := OBMetaNode named: 'markup'.
	
	rootNode
		childAt: #markupFilters
		put: filterNode.
	
	filterNode
		childAt: #markupsByLastActivity
		labeled: 'by last activity'
		put: markupNode;
		childAt: #markupsByChatSize
		labeled: 'by chat size'
		put: markupNode;
		childAt: #markupsByCreationTime
		labeled: 'by creation time'
		put: markupNode;
		childAt: #markupsByTag
		labeled: 'by tag'
		put: markupNode;
		addFilter: OBModalFilter new.
	
	^ (self metaNode: rootNode root: aCodeNode selection: aCodeNode markupFilters first) open! !

!CtMarkupBrowser class methodsFor: 'as yet unclassified' stamp: 'mt 5/30/2009 17:25'!
panels
	^ {self navigationPanel. self definitionPanel} reject: [:ea | ea isNil]! !

!CtMarkupBrowser methodsFor: 'building' stamp: 'mt 5/30/2009 09:24'!
defaultBackgroundColor

	^ Color red muchLighter! !

!CtMarkupBrowser methodsFor: 'accessing' stamp: 'mt 5/30/2009 14:34'!
defaultLabel

	^ 'Markups for {1}' format: {self root markupSourceName}! !

!OBCodeBrowser methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 09:34'!
cmdMarkups

	^ CtCmdBrowseMarkups! !

OBCommand subclass: #CtCmdBrowseMarkups
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-MarkupBrowser'!

!CtCmdBrowseMarkups methodsFor: 'accessing' stamp: 'mt 5/30/2009 18:25'!
buttonLabel

	^ 'markups'! !

!CtCmdBrowseMarkups methodsFor: 'execution' stamp: 'mt 5/30/2009 10:01'!
execute

	CtMarkupBrowser openOn: target copy.! !

!CtCmdBrowseMarkups methodsFor: 'accessing' stamp: 'mt 5/30/2009 09:27'!
group

	^ #codetalk! !

!CtCmdBrowseMarkups methodsFor: 'accessing' stamp: 'mt 5/30/2009 15:19'!
icon

	^ #ctCommentsIcon! !

!CtCmdBrowseMarkups methodsFor: 'accessing' stamp: 'mt 5/30/2009 15:19'!
iconOnly

	^ true! !

!CtCmdBrowseMarkups methodsFor: 'testing' stamp: 'mt 3/18/2010 19:01'!
isActive

	^ target notNil and: [target hasMarkups and: [requestor isSelected: target]]! !

!CtCmdBrowseMarkups methodsFor: 'accessing' stamp: 'mt 5/30/2009 18:25'!
label

	^ 'browse markups'! !

!CtCmdBrowseMarkups methodsFor: 'accessing' stamp: 'mt 5/30/2009 09:28'!
longDescription

	^ 'browse markups'! !

!CtCmdBrowseMarkups methodsFor: 'testing' stamp: 'mt 5/30/2009 09:28'!
wantsButton

	^ true! !

!OBClassNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:55'!
hasMarkups

	^ true! !

!OBClassNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 14:22'!
markupSourceName

	^ '{1} "{2}"' format: {
		self theClass isMeta
			ifTrue: ['meta-class']
			ifFalse: ['class'].
		self theClassName}! !

!OBClassNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 14:13'!
methodReferences

	^ (self theClass selectors collect: [ :each | 
		MethodReference class: self theClass selector: each])! !

!OBClassNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:54'!
referencesForMarkups

	^ self methodReferences! !

OBColumnPanel subclass: #CtMarkupColumnPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-MarkupBrowser'!

!CtMarkupColumnPanel commentStamp: 'mt 6/5/2009 15:53' prior: 0!
Just like the normal OBColumnPanel but uses another building method to get another scroller for special layouting.!

!CtMarkupColumnPanel methodsFor: 'building' stamp: 'mt 6/5/2009 15:53'!
buildOn: aBuilder 
	^aBuilder specialScroller: self with: [columns do: [:ea | ea buildOn: aBuilder]]! !

!OBMethodCategoryNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:07'!
hasMarkups

	^ true! !

!OBMethodCategoryNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 14:29'!
markupSourceName


	^ 'protocol [{1}] in {2} "{3}"' format: {
		self name.
		self theClass isMeta
			ifTrue: ['meta-class']
			ifFalse: ['class'].
		self theClassName}.! !

!OBMethodCategoryNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:47'!
referencesForMarkups

	^ self methodReferences! !

!OBMethodNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:50'!
hasMarkups

	^ true! !

!OBMethodNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 14:28'!
markupSourceName

	^ 'protocol [{1}] in {2} "{3}"' format: {
		self category.
		self theClass isMeta
			ifTrue: ['meta-class']
			ifFalse: ['class'].
		self theClassName}! !

!OBMethodNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:49'!
referencesForMarkups
	"Use markups for all methods in this method's category."
	
	^ (PackageOrganizer default
		packageOfMethod: self reference
		ifNone: [^ #()]) methods select: [:m | m category = self category]! !

!OBMorphBuilder methodsFor: '*ct-markupbrowser' stamp: 'mt 6/5/2009 15:52'!
specialScroller: aColumnPanel with: aBlock 
	| morph |
	morph := (CtPaneScroller withModel: aColumnPanel)
				name: 'scroller';
				vResizing: #spaceFill;
				hResizing: #spaceFill;
				yourself.
	current ifNotNil: [current addMorphBack: morph].
	^self current: morph do: aBlock! !

!OBNode methodsFor: '*ct-markupbrowser' stamp: 'topa 10/27/2009 15:53'!
hasMarkups

	^ false! !

OBCodeNode subclass: #CtMarkupFilterNode
	instanceVariableNames: 'name description markupNodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-MarkupBrowser-Nodes'!

!CtMarkupFilterNode class methodsFor: 'instance creation' stamp: 'mt 6/5/2009 16:04'!
named: aString nodes: someMarkupNodes

	^ self
		named: aString
		nodes: someMarkupNodes
		description: ''! !

!CtMarkupFilterNode class methodsFor: 'instance creation' stamp: 'mt 6/5/2009 16:03'!
named: aString nodes: someMarkupNodes description: anotherString

	^ self new
		name: aString;
		markupNodes: someMarkupNodes;
		description: anotherString! !

!CtMarkupFilterNode methodsFor: 'accessing' stamp: 'mt 6/5/2009 16:03'!
description

	description ifNil: [self description: ''].
	^ description! !

!CtMarkupFilterNode methodsFor: 'accessing' stamp: 'mt 6/5/2009 16:02'!
description: anObject

	description := anObject.! !

!CtMarkupFilterNode methodsFor: 'testing' stamp: 'mt 6/5/2009 15:05'!
hasMarkups

	^ false! !

!CtMarkupFilterNode methodsFor: 'testing' stamp: 'mt 6/5/2009 15:05'!
hasSelector

	^ false! !

!CtMarkupFilterNode methodsFor: 'testing' stamp: 'mt 6/5/2009 15:05'!
hasVersions

	^ false! !

!CtMarkupFilterNode methodsFor: 'ancestry' stamp: 'mt 6/5/2009 16:24'!
isDescendantOfMarkup: anOBCodeNode

	^ true! !

!CtMarkupFilterNode methodsFor: 'accessing' stamp: 'mt 6/5/2009 14:59'!
markupNodes

	^ markupNodes! !

!CtMarkupFilterNode methodsFor: 'accessing' stamp: 'mt 6/5/2009 14:59'!
markupNodes: anObject

	markupNodes := anObject.! !

!CtMarkupFilterNode methodsFor: 'navigation' stamp: 'mt 6/5/2009 15:03'!
markupsByChatSize

	| result groups actualGroup |
	result := OrderedCollection new.
	groups := {
		25 -> '25+ posts'.
		10 -> '10-24 posts'.
		5 -> '5-9 posts'.
		1 -> '1-4 posts'.
		0 -> '(no posts)'}.
	
	self sortMarkupsByChatSize do: [:node | |g|
		g := groups detect: [:s | node attribute chat size >= s key].
		actualGroup = g key ifFalse: [
			result add: (CtSeparatorNode text: g value)].
		actualGroup := g key.
		result add: node].
	^ result! !

!CtMarkupFilterNode methodsFor: 'navigation' stamp: 'mt 6/5/2009 15:03'!
markupsByCreationTime

	| result lastDate |
	result := OrderedCollection new.
	self sortMarkupsByCreationTime do: [:node | |d|
			d := node attribute timestamp date.
			lastDate = d ifFalse: [
				result add: (CtSeparatorNode text: d printRelativeDate)].
			lastDate := d.
			result add: node].
	^ result! !

!CtMarkupFilterNode methodsFor: 'navigation' stamp: 'mt 6/5/2009 15:03'!
markupsByLastActivity

	| result lastDate |
	result := OrderedCollection new.
	self sortMarkupsByLastActivity do: [:node | |d|
			d := node attribute chat
				ifEmpty: [node attribute timestamp date]
				ifNotEmpty: [(node attribute chat last at: #timestamp) date].
			lastDate = d ifFalse: [
				result add: (CtSeparatorNode text: d printRelativeDate)].
			lastDate := d.
			result add: node].
	^ result! !

!CtMarkupFilterNode methodsFor: 'navigation' stamp: 'mt 6/5/2009 15:03'!
markupsByTag

	| result lastTag |
	result := OrderedCollection new.
	self sortMarkupsByTag do: [:node | |t|
			t := node attribute tag.
			lastTag = t ifFalse: [
				result add: (CtSeparatorNode text: ('#{1}' format: {t}))].
			lastTag := t.
			result add: node].
	^ result! !

!CtMarkupFilterNode methodsFor: 'accessing' stamp: 'mt 6/5/2009 16:09'!
name
	"Use the usual placeholder to display the number of nodes."
	
	^ name format: {' (', self markupNodes size asString, ')'}! !

!CtMarkupFilterNode methodsFor: 'accessing' stamp: 'mt 6/5/2009 14:58'!
name: anObject

	name := anObject.! !

!CtMarkupFilterNode methodsFor: 'sorting' stamp: 'mt 6/5/2009 15:02'!
sortMarkupsByChatSize

	^ self markupNodes sortBy: [:n1 :n2 | 
		n1 attribute chat size > n2 attribute chat size]! !

!CtMarkupFilterNode methodsFor: 'sorting' stamp: 'mt 6/5/2009 15:02'!
sortMarkupsByCreationTime

	^ self markupNodes sortBy: [:n1 :n2 |
		n1 attribute timestamp > n2 attribute timestamp]! !

!CtMarkupFilterNode methodsFor: 'sorting' stamp: 'mt 6/5/2009 15:03'!
sortMarkupsByLastActivity

	^ self markupNodes sortBy: [:n1 :n2 | |t1 t2|
		t1 := n1 attribute chat
			ifEmpty: [n1 attribute timestamp]
			ifNotEmpty: [n1 attribute chat last at: #timestamp].
		t2 := n2 attribute chat
			ifEmpty: [n2 attribute timestamp]
			ifNotEmpty: [n2 attribute chat last at: #timestamp].
		t1 > t2]! !

!CtMarkupFilterNode methodsFor: 'sorting' stamp: 'mt 6/5/2009 15:03'!
sortMarkupsByTag

	^ self markupNodes sortBy: [:n1 :n2 |
		n1 attribute tag asString < n2 attribute tag asString]! !

!CtMarkupFilterNode methodsFor: 'public' stamp: 'mt 6/5/2009 18:24'!
text

	| txt |
	txt := self description asText.
	txt addAttribute: (TextColor color: Color gray muchDarker).
	^ txt! !

OBCodeNode subclass: #CtMarkupNode
	instanceVariableNames: 'attribute method snippet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-MarkupBrowser-Nodes'!

!CtMarkupNode class methodsFor: 'instance creation' stamp: 'mt 5/30/2009 10:42'!
forAttribute: aComplexAttribute method: aMethodReference

	^ self new
		attribute: aComplexAttribute;
		method: aMethodReference! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 5/30/2009 10:25'!
attribute

	^ attribute! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 5/30/2009 10:25'!
attribute: anObject

	attribute := anObject.! !

!CtMarkupNode methodsFor: 'public' stamp: 'mt 5/30/2009 17:29'!
browse

	OBCodeBrowser defaultBrowserClass
		openOnClass: self method actualClass
		selector: self method methodSymbol.! !

!CtMarkupNode methodsFor: 'public' stamp: 'mt 6/8/2009 11:23'!
chatMessageFor: aChatLine

	^ self txtChatMessageFor: ('"{1}"'
		format: {aChatLine at: #message}).  ! !

!CtMarkupNode methodsFor: 'public' stamp: 'mt 6/8/2009 10:30'!
chatStampFor: aChatLine

	| txt |
	txt := self txtChatStampFor: ('{1} - {2}' format: {
		(aChatLine at: #timestamp) printRelativeTimeStamp.
		(aChatLine at: #author)}).
		
	"(aChatLine at: #author) = Utilities authorInitials
		ifTrue: [txt append: (self txtHeadFor: ' (You!!)')]."

	^ txt! !

!CtMarkupNode methodsFor: 'public' stamp: 'mt 6/8/2009 10:31'!
creatorFor: aCreatorString

	| txt |
	txt := self txtValueFor: aCreatorString.
		
	"self attribute isMine
		ifTrue: [txt append: (self txtHeadFor: ' (You!!)')]."

	^ txt! !

!CtMarkupNode methodsFor: 'testing' stamp: 'mt 5/30/2009 17:17'!
hasMarkups

	^ false! !

!CtMarkupNode methodsFor: 'testing' stamp: 'mt 5/30/2009 10:26'!
hasSelector

	^ false! !

!CtMarkupNode methodsFor: 'testing' stamp: 'mt 5/30/2009 10:26'!
hasVersions

	^ false! !

!CtMarkupNode methodsFor: 'testing' stamp: 'mt 5/30/2009 17:22'!
isMarkupNode

	^ true! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 5/30/2009 10:41'!
method

	^ method! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 5/30/2009 10:42'!
method: aMethodReference

	method := aMethodReference.! !

!CtMarkupNode methodsFor: 'public' stamp: 'mt 9/22/2010 13:13:30.325'!
name

	| result |
	attribute ifNil: [^ '(no attribute)'].
	result := ('{1}>>{2} (#{3})' format: {
		self theClassName.
		self method methodSymbol.
		self attribute tag}) asText.
	self attribute wasRead ifTrue: [
		result addAttribute: TextColor gray].
	^ result	! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 6/8/2009 11:00'!
snippet

	^ snippet! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 5/30/2009 11:19'!
snippet: anObject

	snippet := anObject.! !

!CtMarkupNode methodsFor: 'public' stamp: 'mt 9/22/2010 13:13:57.26'!
text

	| txt |
	txt := '' asText.
	txt
		append: (self txtHeadFor: 'Creator    ');
		append: (self creatorFor: self attribute creator);
		append: String cr asText;
		append: (self txtHeadFor: 'Timestamp  ');
		append: (self txtValueFor: self attribute timestamp printRelativeTimeStamp, String cr);
		append: (self txtHeadFor: 'Tag        ');
		append: (self txtValueFor: '#', self attribute tag, String cr);
		append: (self txtHeadFor: 'Method     ');
		append: (self txtMethodLinkFor: (self txtValueFor: ('{1}>>{2}' format: {self theClassName. self method methodSymbol}), String cr));
		append: (self txtHeadFor: 'Comment    ');
		append: (self txtCommentFor: ('"{1}"' format: {self attribute comment}), String cr);
		append: (self txtHeadFor: 'Code       ');
		append: (self txtScopeFor: '"', self snippet, '"', String cr);
		append: (self txtSeparator: 50).

	(self attribute chat sortBy: [:line1 :line2 | (line1 at: #timestamp) > (line2 at: #timestamp)])
		do: [:line |
			txt
				append: String cr asText;
				append: (self chatStampFor: line);
				append: String cr asText;
				append: (self chatMessageFor: line);
				append: String cr asText].
	
	self attribute read.
	^ txt! !

!CtMarkupNode methodsFor: 'accessing' stamp: 'mt 9/22/2010 13:13:17.268'!
theClassName

	^ '{1}{2}' format: {
		self method classSymbol.
		self method classIsMeta
			ifTrue: [' class']
			ifFalse: ['']}! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/6/2009 08:46'!
txtChatMessageFor: aString

	^ CtLoader highlightAuthorInitialsFor:
		(self txtMethodLinkFor: (Text string: aString attribute: TextColor black))! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/6/2009 08:55'!
txtChatStampFor: aString

	^ Text string: aString attributes: {TextColor black. TextEmphasis bold}! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/11/2009 11:00'!
txtCommentFor: aString

	^ CtMethodLink addMethodLinksInto:
		(Text
			string: aString
			attributes: {TextColor color: Color green muchDarker})! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/6/2009 08:19'!
txtHeadFor: aString

	^ Text string: aString attribute: TextColor gray! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/6/2009 08:34'!
txtMethodLinkFor: aTextOrString

	^ CtMethodLink addMethodLinksInto: aTextOrString asText! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/8/2009 10:33'!
txtScopeFor: aString

	^ Text string: aString attribute: TextColor black! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/6/2009 09:31'!
txtSeparator: width

	^ Text
		string: ((String new: width) atAllPut: Character space), String cr
		attributes: {TextColor gray. TextEmphasis struckOut}! !

!CtMarkupNode methodsFor: 'formatting' stamp: 'mt 6/6/2009 08:20'!
txtValueFor: aString

	^ Text string: aString attributes: {TextColor black. TextEmphasis bold}! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 6/5/2009 16:07'!
filterAllFor: markupNodes

	^ CtMarkupFilterNode
		named: '-- all{1} --'
		nodes: markupNodes
		description: 'Display all markups in this scope. See the window title to determine the scope.'! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 6/8/2009 10:12'!
filterForCreator: aString for: markupNodes

	^ CtMarkupFilterNode
		named: aString, '{1}'
		nodes: (markupNodes
			select: [:node | node attribute creator = aString])
		description: 'Display only markups that were created by a specific developer. See the window title to determine the current scope.'! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 6/5/2009 16:07'!
filterForMeFor: markupNodes

	^ CtMarkupFilterNode
		named: '-- for me{1} --'
		nodes: (markupNodes
			select: [:node | node attribute isAuthorMentioned])
		description: 'Display only markups for me in this scope. Those are markups with chat messages which include my initials. See the window title to determine the current scope.'! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:05'!
hasMarkups

	^ false! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 6/5/2009 16:20'!
isDescendantOfMarkup: anOBCodeNode

	^ false! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 17:22'!
isMarkupNode

	^ false! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 6/8/2009 10:26'!
markupFilters

	| m creators |
	m := self markups.
	creators := (m collect: [:node | node attribute creator]) asSet asSortedArray.
	^ {self filterAllFor: m. self filterForMeFor: m},
		(creators collect: [:c | self filterForCreator: c for: m]).! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 12:56'!
markupSourceName
	"Return name of markup source, e.g. package name, class category, ..."

	^ self subclassResponsibility! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 1/17/2010 11:55'!
markups

	| result src markups |
	result := OrderedCollection new.
	self referencesForMarkups
		do: [:methodRef |
			markups := CtMarkupDatabase local markupsForMethodReference: methodRef.
			src := methodRef sourceCode asText. "slow..."
			markups do: [:markup |
				result add: ((CtMarkupNode
					forAttribute: markup
					method: methodRef)
					snippet: (markup codeSnippetFor: methodRef sourceCode asString))]]
		displayingProgress: 'Collecting markups...'.
	^ result! !

!OBCodeNode methodsFor: '*ct-markupbrowser' stamp: 'mt 5/30/2009 13:46'!
referencesForMarkups

	^ self subclassResponsibility! !
CtMarkupBrowser initialize!
